﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using IESignal;
namespace BarsChangedDemo
{
    class Program
    {
        static IESignal.Hooks eSignal;

        const string SYMBOL_F = "F";

        static void Main(string[] args)
        {
            InitializeApp();

            // Wait for user to press a key to exit
            Console.ReadKey();
        }



        private static bool InitializeApp()
        {
            bool bSuccess = false;

            eSignal = new IESignal.Hooks();

            try
            {
                // Replace with your account name/logon
                eSignal.SetApplication("<ENTER ACCOUNT NAME>");



                eSignal.OnBarsReceived += new _IHooksEvents_OnBarsReceivedEventHandler(eSignal_OnBarsReceived);
                eSignal.OnBarsChanged += new _IHooksEvents_OnBarsChangedEventHandler(eSignal_OnBarsChanged);
                eSignal.OnQuoteChanged += new _IHooksEvents_OnQuoteChangedEventHandler(eSignal_OnQuoteChanged);

                barType bt = barType.btBARS;

                int iBarsRequested = (int)1440;
                string interval = "1";

                int requestHistoryHandle = eSignal.get_RequestHistory(SYMBOL_F, interval, bt, iBarsRequested, -1, -1);


                bSuccess = requestHistoryHandle > 0;

                eSignal.RequestSymbol(SYMBOL_F, 1);
            }
            catch (Exception ex)
            {
                Console.WriteLine(">>>>>>>>>>>>InitializeApp EXCEPTION:{0}", ex.Message);
            }
            return bSuccess;
        }

        #region eSignal event handlers
        static void eSignal_OnQuoteChanged(string sSymbol)
        {
            try
            {
                BasicQuote q = eSignal.get_GetBasicQuote(sSymbol);

                Console.WriteLine(string.Format("eSignal_OnQuoteChanged:{0}:Quote received for symbol: {1}  LAST:{2}",
                    DateTime.Now.ToLongTimeString(),
                    sSymbol,
                    q.dLast
                ));
            }
            catch (Exception ex)
            {
                Console.WriteLine("eSignal_OnQuoteChanged(): EXCEPTION:{0}", ex.Message);
            }
        }

        static void eSignal_OnBarsChanged(int lHandle)
        {
            try
            {
                Console.WriteLine("eSignal_OnBarsChanged:{1} BarsChanged signal received for Handle {0}", lHandle, DateTime.Now.ToLongTimeString());

                int numBars = eSignal.get_GetNumBars(lHandle);

                IESignal.BarData currbar = eSignal.get_GetBar(lHandle, 0);
                Console.WriteLine(
                    string.Format("BAR DATA: Time:{0}  Close:{1}",
                    currbar.dtTime,
                    currbar.dClose));
            }
            catch (Exception ex)
            {
                Console.WriteLine(">>>>>>>>>>>>eSignal_OnBarsChanged EXCEPTION:" + ex.Message);
            }
            
        }

        static void eSignal_OnBarsReceived(int lHandle)
        {
            try
            {
                while (eSignal.get_IsHistoryReady(lHandle) == 0)
                {
                    System.Threading.Thread.Sleep(100);
                }

                System.Threading.ThreadPool.QueueUserWorkItem(new System.Threading.WaitCallback(ProcessHistory), lHandle);
            }
            catch (Exception ex)
            {
                Console.WriteLine(">>>>>>>>>>>> eSignal_OnBarsReceived EXCEPTION: " + ex.Message);
            }
        }

       

        #endregion

        static void ProcessHistory(object oObjectState)
        {
            int iHandle = (int)oObjectState;

            try
            {
                ProcessBardataHandle(iHandle);
            }
            catch (Exception ex)
            {
                Console.WriteLine(">>>>>>>>>>>>>>ProcessHistory() EXCEPTION:" + ex.Message);
            }

        }

        static private void ProcessBardataHandle(int iHandle)
        {
            Console.WriteLine("ProcessBardataHandle ENTER");

            int numBars = eSignal.get_GetNumBars(iHandle);
            try
            {
                try
                {
                    int iLast = (-1 * numBars);
                    for (int i = 0; i > iLast; i--)
                    {
                        try
                        {
                            BarData currbar = eSignal.get_GetBar(iHandle, i);
                            if (currbar.dtTime.Date >= DateTime.Now.Date)
                                continue;

                            Console.WriteLine("ProcessBardataHandle: BarTime:{0}\tClose{1}", currbar.dtTime, currbar.dClose);

                        }
                        catch (Exception ex)
                        {
                            Console.WriteLine("ProcessBardataHandle:EXCEPTION:{0}", ex.Message);
                        }
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine("ProcessBardataHandle:EXCEPTION:{0}", ex.Message);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(">>>>>>>> EXCEPTION: " + ex.Message);
            }
            finally
            {
               
                eSignal.ReleaseHistory(iHandle);
            }
        }

    }
}
